@@ -3,24 +3,34 @@ import numpy as np
 import os
 import h5py
 import matplotlib.pyplot as plt
+from einops import repeat
 from . import source
 from .source import filter_mag, read_filter
+from .config import get_throughput
 import pdb
 
+path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
+refdata = os.path.join(path, 'refdata/')
 
-refdata = os.path.abspath(os.path.dirname(__file__)+'../refdata') + '/'
 
-def flux2electron(config, wavearr, fluxarr, throughtputwave, throughtputvalue, fluxunit=''):
-    '''
-    :param wavearr: A
-    :param fluxarr: erg/s/cm^2/A
-    :return:
-    '''
+def flux2electron(spaxel_xsize, spaxel_ysize, wavearr, fluxarr, throughtputwave, throughtputvalue, fluxunit=''):
+    """
 
-    instru = config['configuration']
+    Args:
+        spaxel_xsize:
+        spaxel_ysize:
+        wavearr:
+        fluxarr: erg/s/cm^2/A/arcsec^2 or erg/s/cm^2/Hz/arcsec^2
+        throughtputwave:
+        throughtputvalue:
+        fluxunit: 'erg/s/cm^2/A' or 'erg/s/cm^2/hz'
+
+    Returns:
+
+    """
 
     cc = 3.0e18   # speed of light, A/s
-    spaxel_area = instru['spaxel_xsize'] * instru['spaxel_ysize']    # arcsec^2
+    spaxel_area = spaxel_xsize * spaxel_ysize    # arcsec^2
     d = 200  # diameter of the telescope, in cm unit
     obscure = 0.0  # effective central obscuration, no unit
     telarea = 3.14159 / 4.0 * d * d * (1.0 - obscure)  # effective area of the telescope, cm^2
@@ -31,7 +41,15 @@ def flux2electron(config, wavearr, fluxarr, throughtputwave, throughtputvalue, f
     hv = planckh * cc / wavearr  # erg
     QE = 1
 
-    qvalue = np.interp(wavearr, throughtputwave, throughtputvalue)
+    if wavearr.ndim == 3:
+        qvalue = np.zeros(shape=wavearr.shape, dtype=float)
+        qvalue = np.interp(wavearr[:, 0, 0], throughtputwave, throughtputvalue)
+        qvalue = repeat(qvalue, 'h -> h '+str(wavearr.shape[1])+' '+str(wavearr.shape[2]))
+    elif wavearr.ndim == 1:
+        qvalue = np.interp(wavearr, throughtputwave, throughtputvalue)
+    else:
+        raise ValueError('check dimension of the input array.')
+
 
     if fluxunit=='erg/s/cm^2/A':
         isource1 = fluxarr * wavearr ** 2 / cc * spaxel_area  # erg/s/cm^2/Hz per spaxel
@@ -87,24 +105,22 @@ def electron2flux(config, wavearr, countsarr, throughtputwave, throughtputvalue,
         return np.nan
 
 
-def get_throughput(config):
-
-    throughput = pd.read_csv(os.getenv('SNPP_PATH') + 'refdata/csst/ifs/IFU_throughput.dat',
-                             sep='\s+', skiprows=1, header=None, names=['nm', 'q'])
-    throughputwave = throughput.nm.values * 10  # nm to A
-    qtot = throughput.q.values  # ; throughput of the whole system,
-
-    # ;assuming the total throughput cannot reach the theory value, 0.3 is the upper limit.
-    qtot[qtot >= 0.3] = 0.3
-
-    factor = 1.0                    # in case the efficiency could be lower
-    throughputvalue = qtot * factor  # qtot of CSST already includes the CCD efficiency
-
-    return throughputwave, throughputvalue
-
 
 def get_nsky(wavearr):
 
+    """
+
+    Args:
+        wavearr:
+        spaxel_xsize:
+        spaxel_ysize:
+
+    Returns:
+        sky counts (e-/s), in 1d array.
+
+
+    """
+
     # know the sky counts in MS ccd per pixel in different bands,
     # Thus convert it to sky spectrum (counts) in IFS ccd per spaxel
 
@@ -136,7 +152,57 @@ def get_nsky(wavearr):
     return fluxskypp
 
 
-class DetectorSignal(source.ModelCube):
+def get_background(zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_ysize):
+
+    """
+    Consider zodiacal background and earth shine background in different levels.
+
+    Args:
+        zodia_level:
+            zodiacal level, low, median, high, corresponding index from 1-3.
+        earthshine_level:
+            earth shine level, low, median, high, corresponding index from 1-3.
+        wavearr:
+            wave array to be returned.
+
+    Returns:
+        sky counts (e-/s) in each spaxel element (spaxel_xsize * spaxel_ysize), in 1d array.
+
+    """
+
+    # get IFS throughtput
+    throughtputwave, throughtputvalue = get_throughput()
+
+    # get the zodia spectrum, convert to e/s/spaxel/spec-elements
+    filename = os.path.join(refdata, 'csst', 'background', 'zodi_spec.csv')
+    cat = pd.read_csv(filename, comment='#')
+    zodia_wave = cat['wave']
+    zodia_spec = cat['zodi_' + zodia_level]       # erg/s/cm^2/A/arcsec^2
+    zodia_spec_interp = np.interp(wavearr, zodia_wave, zodia_spec)
+    zodia_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
+                                       wavearr, zodia_spec_interp,
+                                       throughtputwave, throughtputvalue,
+                                       fluxunit='erg/s/cm^2/A')
+
+    # get the earth shine spectrum, convert to e/s/spaxel/spec-elements
+    filename = os.path.join(refdata, 'csst', 'background', 'earthshine_spec.csv')
+    cat = pd.read_csv(filename, comment='#')
+    earthshine_wave = cat['wave']
+    earthshine_spec = cat['earthshine_' + earthshine_level]       # erg/s/cm^2/A/arcsec^2
+    earthshine_spec_interp = np.interp(wavearr, earthshine_wave, earthshine_spec)
+    earthshine_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
+                                       wavearr, earthshine_spec_interp,
+                                       throughtputwave, throughtputvalue,
+                                       fluxunit='erg/s/cm^2/A')
+
+    nsky = zodia_counts_interp + earthshine_counts_interp
+
+
+    return nsky
+
+
+
+class DetectorCountsRate(source.ModelCube):
 
     def __init__(self, config):
 
@@ -148,16 +214,18 @@ class DetectorSignal(source.ModelCube):
             self, config
         )
 
-        throughtputwave, throughtputvalue = get_throughput(config)
+        throughtputwave, throughtputvalue = get_throughput()
 
-        self.source_counts = flux2electron(config, self.wavecube, self.fluxcube,
-                                throughtputwave, throughtputvalue,
-                                fluxunit='erg/s/cm^2/A')
+        instru = config['configuration']
+        self.source_counts = flux2electron(instru['spaxel_xsize'], instru['spaxel_ysize'],
+                                           self.wavecube, self.fluxcube,
+                                           throughtputwave, throughtputvalue,
+                                           fluxunit='erg/s/cm^2/A')
         self.wavearr = self.wavecube
 
 
 
-class DetectorNoise(object):
+class DetectorCounts(object):
 
     def __init__(self, config, signal):
 
@@ -180,12 +248,13 @@ class DetectorNoise(object):
         self.skynn = (nskycube * tot_t)  # e/s/sampling * t per spaxel
 
 
+
 class calculation_snr(object):
 
     def __init__(self, config, straylight=False):
 
-        signal_rate = DetectorSignal(config)
-        noise = DetectorNoise(config, signal_rate)
+        signal_rate = DetectorCountsRate(config)
+        noise = DetectorCounts(config, signal_rate)
 
         self.source = noise.sourcenn
         self.darknoise = np.sqrt(noise.darkn)
@@ -377,7 +446,7 @@ class calculation_exptime(object):
         npixw = 2
         specsampling = 1
 
-        signal_rate = DetectorSignal(config)
+        signal_rate = DetectorCountsRate(config)
         isource = signal_rate.source_counts
 
         # need to do loop, to limit the continous time per orbit and solve the repeat number.
@@ -419,6 +488,7 @@ def perform_calculation(config, calculation_mode='exptime2snr'):
     else:
 
         print('calculation_mode need to be " exptime2snr | snr2exptime | snr2limitmag "')
+        report = None
         pdb.set_trace()
 
 
