@@ -3,39 +3,123 @@ import pandas as pd
 import os
 import astropy.io.fits as fits
 from scipy.integrate import simps
+import extinction
 import pdb
 
-refdata = os.path.abspath(os.path.dirname(__file__)+'/../refdata') + '/'
+path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
+refdata = os.path.join(path, 'refdata/')
+
 
 
 class read_template(object):
 
-    def __init__(self, source):
+    r"""
+    get spectrum template using the input parameters
 
-        self.source = source
-        self.line_definitions = self.source['lines']
+    .. todo::
+        enable to add emission lines
+
+
+    Args:
+        isource (dict, including 3 keys):
+            name:
+                the spectrum filename
+            redshift:
+                the redshift applied to the template
+            ebv:
+                the extinction applied to the template
+
+    Attributes:
+
+        wave (`numpy.ndarray`, unit in Angstrom):
+            the wavelength of the spectrum, redshift applied.
+        flux (`numpy.ndarray`, unit in erg/s/cm^2/A)
+            the flux of the spectrum, redshift and extinction considered.
+
+    """
+
+    def __init__(self, isource):
+
+        # self.source = isource
+        # self.line_definitions = self.source['lines']
 
         # snpp_path = '/Users/linlin/Pro/snpp_v1_05/refdata/'
-        template_filename = refdata + 'sed/' + self.source['name']
+        # template_filename = snpp_refdata + 'sed/' + self.source['name']
         # filtera = snpp_path + 'normalization/filters/sdss_g0.par'
         # result = input_mag_model(self.source['normalization']['value'],
         #                          galtpl, filtera)  # scale the template to given brightness
-        hdu = fits.open(template_filename)
-        self.wave = hdu[1].data['wavelength']   # A
-        self.flux = hdu[1].data['flux'] * 1e-12         # 10^-12 erg/s/A/cm2
+
+        template_filename = isource['name']
+        template_redshift = isource['redshift']
+        template_ebv = isource['ebv']
+
+        if template_filename.endswith('.fits'):
+
+            if template_filename in ['SFgal_texp_FeH0_tau5_Ew10_AGN1.fits',
+                                     'SFgal_texp_FeH0_tau1_Ewd.fits',
+                                     'SFgal_texp_FeH0_tau5_Ew5.fits',
+                                     'SFgal_texp_FeH-2_tau10_Ew50.fits']:
+                template_filename = os.path.join(refdata, 'sed', template_filename)
+
+            hdu = fits.open(template_filename)
+            template_wave = hdu[1].data['wavelength']   # A
+            template_flux = hdu[1].data['flux'] * 1e-12         # erg/s/A/cm2
+
+            # If the data is big endian, swap the byte order to make it little endian
+            if template_wave.dtype.byteorder == '>':
+                template_wave = template_wave.byteswap().newbyteorder()
+
+        elif template_filename.endswith('.txt'):
+
+            cat = pd.read_csv(template_filename, sep='\s+', header=None)
+            template_wave = cat[0]      # unit should be in Angstrom
+            template_flux = cat[1]      # unit should be in erg/s/cm^2/A
+
+        else:
+            raise ValueError("File name need to be ended with '.fits' or '.txt'. ")
+
+        # extinction require the dtype should be double.
+        template_wave = np.array(template_wave, dtype='float64')
+
+        # extinction correction
+        # assuming Calzetti-00 extinction law.
+        r_v = 4.5
+        extinction_mag = extinction.calzetti00(template_wave,
+                                               template_ebv * r_v,
+                                               r_v, unit='aa')
+        template_flux = extinction.apply(extinction_mag, template_flux)
+
+        # redshift correction
+        template_wave = template_wave * (1 + template_redshift)
+        template_flux = template_flux / (1 + template_flux)
+
+        self.wave = template_wave
+        self.flux = template_flux
+
+
+def get_wavearr(config):
+
+    instr_par = config['configuration']
+    wave_start = instr_par['wave_start']
+    wave_end = instr_par['wave_end']
+    wave_delta = instr_par['ccd_xsize'] * instr_par['readout_xbin']
+
+    ccdspec_wave = np.arange(wave_start, wave_end, wave_delta)
+
+    return ccdspec_wave
 
 
 class read_filter(object):
-    # load the filters
-    # filterfile = filtera  # '../sdss_g0.par'
-    # filterpath='./'
-    # filterfile=filterpath+filtersel   # ;fluxfilter: max=1, min=0, no particular unit
-    def __init__(self, band):
 
-        filterfile = refdata + 'normalization/filters/sdss_g0.par'
-        # print('Filter File:', filterfile)
+    def __init__(self, filter_name=''):
+        """
 
-        band = pd.read_csv(filterfile, sep='\s+', header=None, comment='#')
+        Args:
+            filter_name:
+        """
+
+        filter_file = os.path.join(refdata, 'normalization', 'filters', filter_name+'.par')
+        band = pd.read_csv(filter_file, sep='\s+', header=None, comment='#')
         self.wave = band[0].values  # A
         self.throughput = band[1].values  # vaccum_pass
         self.wavemin = self.wave[0]
@@ -55,29 +139,36 @@ class read_filter(object):
 
 
 def filter_mag(objwave, objflux, filterwave, filterthroughtput, output='mag'):
-    '''
-    :param objwave: unit as A
-    :param objflux: unit as erg/s/cm^2/A
-    :param filterwave: unit as A
-    :param filterthroughtput: unit as detector signal per photon (use vaccum_pass for space telescope,
+    """
+    Get AB magnitude in a certain filter.
+
+    Args:
+        objwave: unit as A
+        objflux: unit as erg/s/cm^2/A
+        filterwave: unit as A
+        filterthroughtput: unit as detector signal per photon (use vaccum_pass for space telescope,
                               otherwise select a given airmass)
-    :return: AB magnitude in this band
-    '''
+
+    Return:
+        AB magnitude in this band
+
+    """
 
     # resample the throughtput to objwave
     ind = (objwave >= np.min(filterwave)) & (objwave <= np.max(filterwave))
     wavetmp = objwave[ind]
     phot_frac = np.interp(wavetmp, filterwave, filterthroughtput)  # phot fraction (/s/A/cm^2?)
+
     # convert to energy fraction
     # E_frac = E0/hv * N_frac / E0 ~ N_frac/nu ~ N_frac * lambda
     energy_frac = phot_frac * wavetmp  # convert to energy fraction
 
     # convert the objflux to erg/s/cm^2/Hz
-    c = 3e18    # A/s
-    objflux_hz = objflux[ind] * (objwave[ind]**2) / c    # erg/s/scm^2/Hz
+    c = 3e18  # A/s
+    objflux_hz = objflux[ind] * (objwave[ind] ** 2) / c  # erg/s/scm^2/Hz
 
     from scipy.integrate import simps
-    integrate_flux = simps(objflux_hz * energy_frac, c/wavetmp) / simps(energy_frac, c/wavetmp)
+    integrate_flux = simps(objflux_hz * energy_frac, c / wavetmp) / simps(energy_frac, c / wavetmp)
 
     if output == 'mag':
         mag = -2.5 * np.log10(integrate_flux) - 48.6
@@ -89,9 +180,8 @@ def filter_mag(objwave, objflux, filterwave, filterthroughtput, output='mag'):
         return np.nan
 
 
-def normalized(template_wave, template_flux, config):
 
-    # grid = get_grid(config)
+def normalized(template_wave, template_flux, config):
 
     source = config['source']
 
@@ -111,43 +201,13 @@ def normalized(template_wave, template_flux, config):
     u0 = 10**((u0 + 48.6)/(-2.5))         # target flux in erg/s/cm^2/Hz unit
     u0 = u0 * 3e18 / filter.waveeff**2    # erg/s/cm^2/A
 
-    # geometry = source['geometry']
-    # if geometry['type'] == 'Sersic':
-    #
-    #     xcen = grid.nx / 2 + geometry['x_offset']
-    #     ycen = grid.ny / 2 + geometry['y_offset']
-    #     rmaj = geometry['Re_major'] / grid.xspxsize
-    #     rmin = geometry['Re_minor'] / grid.xspxsize
-    #
-    #     scaled_img = sersic(grid.nx, grid.ny, xcen, ycen,
-    #              u0, rmaj, rmin/rmaj, geometry['PA'], geometry['sersic_n'])
-    #
-    # elif geometry['type'] == 'flat':
-    #
-    #     xcen = grid.nx / 2 + geometry['x_offset']
-    #     ycen = grid.ny / 2 + geometry['y_offset']
-    #
-    #     scaled_img = flatbox(grid.nx, grid.ny, xcen, ycen,
-    #                     u0, geometry['xwidth'], geometry['ywidth'])
-    #
-    # else:
-    #     print('geometry should be one of these: ("Sersic" | "flat").')
-    #     pdb.set_trace()
-
     factor = u0 * filter_constant / template_constant
 
-    # norm_wave = np.zeros(shape=(len(template_wave), grid.ny, grid.nx), dtype=float)
-    # norm_flux = np.zeros(shape=(len(template_wave), grid.ny, grid.nx), dtype=float)
-    # for j in range(grid.ny):
-    #     for i in range(grid.nx):
     norm_wave = template_wave                  # A
     norm_flux = template_flux * factor         # erg/s/cm^2/A
 
 
     # check the normalized magnitude should be normalized.value - 2.5*log(0.04)
-    # mag2d = np.zeros(shape=(grid.ny, grid.nx), dtype=float)
-    # for j in range(grid.ny):
-    #     for i in range(grid.nx):
     mag = filter_mag(norm_wave, norm_flux,
                      filter_wave, filter_flux, output='mag')
 
@@ -159,9 +219,10 @@ class ModelCube(object):
 
     def __init__(self, config):
 
-        self.ccdspec_wave = np.arange(3500, 10000, 1.75555)
+        self.ccdspec_wave = get_wavearr(config)
         self.ccdspec_nw = len(self.ccdspec_wave)
 
+
         # grid = get_grid(config)
 
         object = config['source']['spectrum']
