@@ -9,14 +9,24 @@ from . import source
 from .source import filter_mag, read_filter
 from .config import get_throughput
 
-snpp_refdata = os.path.abspath(os.path.dirname(__file__)+'/../refdata') + '/'
+path = os.path.dirname(os.path.abspath(os.path.dirname(__file__)))
+snpp_refdata = os.path.join(path, 'refdata/')
 
 def flux2electron(spaxel_xsize, spaxel_ysize, wavearr, fluxarr, throughtputwave, throughtputvalue, fluxunit=''):
-    '''
-    :param wavearr: A
-    :param fluxarr: erg/s/cm^2/A/arcsec^2
-    :return:
-    '''
+    """
+
+    Args:
+        spaxel_xsize: 
+        spaxel_ysize: 
+        wavearr: 
+        fluxarr: erg/s/cm^2/A/arcsec^2 or erg/s/cm^2/Hz/arcsec^2
+        throughtputwave: 
+        throughtputvalue: 
+        fluxunit: 'erg/s/cm^2/A' or 'erg/s/cm^2/hz'
+
+    Returns:
+
+    """
 
     # instru = config['configuration']
 
@@ -97,6 +107,19 @@ def electron2flux(config, wavearr, countsarr, throughtputwave, throughtputvalue,
 
 def get_ms_nsky(wavearr, spaxel_xsize, spaxel_ysize):
 
+    """
+
+    Args:
+        wavearr:
+        spaxel_xsize:
+        spaxel_ysize:
+
+    Returns:
+        sky counts (e-/s), in 1d array.
+
+
+    """
+
     # know the sky counts in MS ccd per pixel in different bands,
     # Thus convert it to sky spectrum (counts) in IFS ccd per spaxel
 
@@ -128,35 +151,51 @@ def get_ms_nsky(wavearr, spaxel_xsize, spaxel_ysize):
     return fluxskypp
 
 
-def get_background(background_type, background_level, wavearr, spaxel_xsize, spaxel_ysize):
 
-    # output: sky counts (e-/s), in 1d array.
+def get_background(zodia_level, earthshine_level, wavearr, spaxel_xsize, spaxel_ysize):
 
-    if background_type == 'from_msc':
-        nsky = get_ms_nsky(wavearr, spaxel_xsize, spaxel_ysize)
+    """
+    Consider zodiacal background and earth shine background in different levels.
 
-    elif background_type == 'zodi':
+    Args:
+        zodia_level:
+            zodiacal level, low, median, high, corresponding index from 1-3.
+        earthshine_level:
+            earth shine level, low, median, high, corresponding index from 1-3.
+        wavearr:
+            wave array to be returned.
 
-        # get IFS throughtput
-        throughtputwave, throughtputvalue = get_throughput()
+    Returns:
+        sky counts (e-/s) in each spaxel element (spaxel_xsize * spaxel_ysize), in 1d array.
 
-        # get the zodi spectrum from Leinert98, convert to e/s/spaxel/spec-elements
-        cat = pd.read_csv(os.getenv('SNPP_PATH') + 'refdata/csst/background/zodi_spec_L98.csv', comment='#')
-        zodi_wave = cat['wave']
-        if background_level in ['high', 'med', 'low']:
-            zodi_spec = cat['zodi_' + background_level] / 4.25452e10  # erg/s/cm^2/A/arcsec^2
-            zodi_spec_interp = np.interp(wavearr, zodi_wave, zodi_spec)
-            zodi_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
-                                               wavearr, zodi_spec_interp,
-                                               throughtputwave, throughtputvalue,
-                                               fluxunit='erg/s/cm^2/A')
-        else:
-            raise ValueError("background_type need to be '(high | med | low). ")
+    """
 
-        nsky = zodi_counts_interp
+    # get IFS throughtput
+    throughtputwave, throughtputvalue = get_throughput()
 
-    else:
-        raise TypeError("background_type need to be 'from_msc' or 'zodi'. ")
+    # get the zodia spectrum, convert to e/s/spaxel/spec-elements
+    filename = os.path.join(snpp_refdata, 'csst', 'background', 'zodi_spec.csv')
+    cat = pd.read_csv(filename, comment='#')
+    zodia_wave = cat['wave']
+    zodia_spec = cat['zodi_' + zodia_level]       # erg/s/cm^2/A/arcsec^2
+    zodia_spec_interp = np.interp(wavearr, zodia_wave, zodia_spec)
+    zodia_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
+                                       wavearr, zodia_spec_interp,
+                                       throughtputwave, throughtputvalue,
+                                       fluxunit='erg/s/cm^2/A')
+
+    # get the earth shine spectrum, convert to e/s/spaxel/spec-elements
+    filename = os.path.join(snpp_refdata, 'csst', 'background', 'earthshine_spec.csv')
+    cat = pd.read_csv(filename, comment='#')
+    earthshine_wave = cat['wave']
+    earthshine_spec = cat['earthshine_' + earthshine_level]       # erg/s/cm^2/A/arcsec^2
+    earthshine_spec_interp = np.interp(wavearr, earthshine_wave, earthshine_spec)
+    earthshine_counts_interp = flux2electron(spaxel_xsize, spaxel_ysize,
+                                       wavearr, earthshine_spec_interp,
+                                       throughtputwave, throughtputvalue,
+                                       fluxunit='erg/s/cm^2/A')
+
+    nsky = zodia_counts_interp + earthshine_counts_interp
 
 
     return nsky
@@ -203,12 +242,15 @@ class DetectorCounts(object):
 
         nreadout_rate = np.zeros(shape=nw, dtype=float) + instru['readout_noise']**2    # e/pix
 
-        nsky_rate = get_background(config['background_type'], config['background_level'],
-                                 signal.ccdspec_wave, instru['spaxel_xsize'], instru['spaxel_ysize'])
+        nsky_rate = get_background(config['background']['zodiacal'],
+                                   config['background']['earth_shine'],
+                                   signal.ccdspec_wave,
+                                   instru['spaxel_xsize'],
+                                   instru['spaxel_ysize'])
 
         # read the exptime
-        repn = config['repn']
-        obst = config['obst']
+        repn = config['calc_mode']['repn']
+        obst = config['calc_mode']['obst']
         npixw = 2
         specsampling = 1
 
@@ -291,6 +333,7 @@ class calculation_snr(object):
     def __init__(self, config):
 
         signal_rate = DetectorCountsRate(config)
+
         counts = DetectorCounts(config, signal_rate)
 
         self.source = counts.nsource
@@ -522,16 +565,20 @@ class read_report(object):
 
 
 
-def perform_calculation(config, calculation_mode='exptime2snr'):
+def perform_calculation(config):
 
 
-    if calculation_mode == 'exptime2snr':
+    calculation_mode = config['calc_mode']['mode']
+
+    if calculation_mode == 1:       # known the target SNR, calculate the needed time.
+
+        # report = calculation_exptime(config)
+        report = None
+        print('This calc_mode is under development. Return none.')
+
+    elif calculation_mode == 2:     # known the exposure time, calculate the reached SNR.
 
         report = calculation_snr(config)
-
-    # elif calculation_mode == 'snr2exptime':
-    #
-    #     report = calculation_exptime(config)
     #
     # elif calculation_mode == 'snr2limitmag':
     #
@@ -539,8 +586,9 @@ def perform_calculation(config, calculation_mode='exptime2snr'):
 
     else:
 
-        print('calculation_type need to be " exptime2snr | snr2exptime | snr2limitmag "')
-        pdb.set_trace()
+        report = None
+        print('This calc_mode is not defined. ')
+
 
 
     return report
